@@ -4,10 +4,11 @@ use strict;
 use warnings;
 
 use Chemistry::OpenSMILES qw(is_aromatic);
+use Chemistry::OpenSMILES::Parser;
 use Graph::Traversal::DFS;
 
 # ABSTRACT: OpenSMILES format writer
-our $VERSION = '0.4.2'; # VERSION
+our $VERSION = '0.4.4'; # VERSION
 
 require Exporter;
 our @ISA = qw( Exporter );
@@ -35,6 +36,7 @@ sub write_SMILES
         my %vertex_symbols;
         my $nrings = 0;
         my %seen_rings;
+        my @chiral;
 
         my $rings = {};
 
@@ -43,6 +45,7 @@ sub write_SMILES
                                        @_ = ( $_[1], $_[0], $_[2] );
                                    }
                                    push @symbols, _tree_edge( @_ ) },
+
             non_tree_edge => sub { my @sorted = sort { $vertex_symbols{$a} <=>
                                                        $vertex_symbols{$b} }
                                                      @_[0..1];
@@ -50,12 +53,18 @@ sub write_SMILES
                                            {$vertex_symbols{$sorted[1]}} =
                                         _depict_bond( @sorted, $graph ); },
 
-            pre  => sub { my( $vertex, $graph ) = @_;
+            pre  => sub { my( $vertex, $dfs ) = @_;
+                          if( $vertex->{chirality} &&
+                              $vertex->{chirality} =~ /^@@?$/ &&
+                              $dfs->graph->degree( $vertex ) == 4 ) {
+                              push @chiral, $vertex;
+                          }
                           push @symbols,
-                         _pre_vertex( { map { $_ => $vertex->{$_} }
-                                        grep { $_ ne 'chirality' }
-                                        keys %$vertex }, $graph );
+                          _pre_vertex( { map { $_ => $vertex->{$_} }
+                                         grep { $_ ne 'chirality' }
+                                         keys %$vertex } );
                           $vertex_symbols{$vertex} = $#symbols },
+
             post => sub { push @symbols, ')' },
         };
 
@@ -72,6 +81,24 @@ sub write_SMILES
         next unless @symbols;
         pop @symbols;
 
+        # Dealing with chirality
+        for my $atom (@chiral) {
+            my @neighbours = map { $_->{number} }
+                             sort { $vertex_symbols{$a} <=>
+                                    $vertex_symbols{$b} }
+                             $graph->neighbours($atom);
+            my $chirality_now = _tetrahedral_chirality( $atom->{chirality},
+                                                        @neighbours );
+            my $parser = Chemistry::OpenSMILES::Parser->new;
+            my( $graph_reparsed ) = $parser->parse( $symbols[$vertex_symbols{$atom}],
+                                                    { raw => 1 } );
+            my( $atom_reparsed ) = $graph_reparsed->vertices;
+            $atom_reparsed->{chirality} = $chirality_now;
+            $symbols[$vertex_symbols{$atom}] =
+                write_SMILES( $atom_reparsed );
+        }
+
+        # Adding ring numbers
         my @ring_ids = ( 1..99, 0 );
         my @ring_ends;
         for my $i (0..$#symbols) {
@@ -169,6 +196,38 @@ sub _depict_bond
     return $bond eq '/' ? '\\' : '/';
 }
 
+# Invert the tetrahedral chirality sign if the order of attachments
+# has changed from clockwise to counter-clockwise and vice versa.
+sub _tetrahedral_chirality
+{
+    my( $chirality, @numbers ) = @_;
+
+    # Translating the numbers to range of 0..3
+    my @indices = sort { $numbers[$a] <=> $numbers[$b] } 0..3;
+    foreach (0..3) {
+        $numbers[$indices[$_]] = $_;
+    }
+
+    # First attachment is written on left hand side of the chiral
+    # center, here it is called $direction
+    my $direction = shift @numbers;
+
+    # Cyclically sorting the rest of the attachments
+    while( $numbers[0] > $numbers[1] || $numbers[0] > $numbers[2] ) {
+        push @numbers, shift @numbers;
+    }
+
+    # Checking whether the direction has been changed
+    if( ($direction == 0 && $numbers[1] == 2) ||
+        ($direction == 1 && $numbers[1] == 3) ||
+        ($direction == 2 && $numbers[1] == 1) ||
+        ($direction == 3 && $numbers[1] == 2) ) {
+        return $chirality;
+    } else {
+        return $chirality eq '@' ? '@@' : '@';
+    }
+}
+
 sub _order
 {
     my( $vertices ) = @_;
